@@ -14,10 +14,10 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 #include "netdissect.h"
 #include "l2vpn.h"
 
@@ -26,7 +26,7 @@
  *
  * RFC 6624
  *
- * http://www.iana.org/assignments/bgp-parameters/bgp-parameters.xhtml#bgp-l2-encapsulation-types-registry
+ * https://www.iana.org/assignments/bgp-parameters/bgp-parameters.xhtml#bgp-l2-encapsulation-types-registry
  */
 const struct tok l2vpn_encaps_values[] = {
     { 0, "Reserved"},
@@ -61,7 +61,7 @@ const struct tok l2vpn_encaps_values[] = {
  *
  * RFC 4446
  *
- * http://www.iana.org/assignments/pwe3-parameters/pwe3-parameters.xhtml#pwe3-parameters-2
+ * https://www.iana.org/assignments/pwe3-parameters/pwe3-parameters.xhtml#pwe3-parameters-2
  */
 const struct tok mpls_pw_types_values[] = {
     { 0x0000, "Reserved"},
