@@ -39,13 +39,17 @@
 /* \summary: ASCII packet dump printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
+
 #include <stdio.h>
 
+#include "netdissect-ctype.h"
+
 #include "netdissect.h"
+#include "extract.h"
 
 #define ASCII_LINELENGTH 300
 #define HEXDUMP_BYTES_PER_LINE 16
@@ -59,14 +63,16 @@ ascii_print(netdissect_options *ndo,
             const u_char *cp, u_int length)
 {
 	u_int caplength;
-	register u_char s;
+	u_char s;
 
-	caplength = (ndo->ndo_snapend >= cp) ? ndo->ndo_snapend - cp : 0;
+	ndo->ndo_protocol = "ascii";
+	caplength = (ndo->ndo_snapend > cp) ? ND_BYTES_AVAILABLE_AFTER(cp) : 0;
 	if (length > caplength)
 		length = caplength;
-	ND_PRINT((ndo, "\n"));
+	ND_PRINT("\n");
 	while (length > 0) {
-		s = *cp++;
+		s = GET_U_1(cp);
+		cp++;
 		length--;
 		if (s == '\r') {
 			/*
@@ -78,72 +84,76 @@ ascii_print(netdissect_options *ndo,
 			 *
 			 * In the middle of a line, just print a '.'.
 			 */
-			if (length > 1 && *cp != '\n')
-				ND_PRINT((ndo, "."));
+			if (length > 1 && GET_U_1(cp) != '\n')
+				ND_PRINT(".");
 		} else {
-			if (!ND_ISGRAPH(s) &&
+			if (!ND_ASCII_ISGRAPH(s) &&
 			    (s != '\t' && s != ' ' && s != '\n'))
-				ND_PRINT((ndo, "."));
+				ND_PRINT(".");
 			else
-				ND_PRINT((ndo, "%c", s));
+				ND_PRINT("%c", s);
 		}
 	}
 }
 
-void
-hex_and_ascii_print_with_offset(netdissect_options *ndo, register const char *ident,
-    register const u_char *cp, register u_int length, register u_int oset)
+static void
+hex_and_ascii_print_with_offset(netdissect_options *ndo, const char *ident,
+    const u_char *cp, u_int length, u_int oset)
 {
 	u_int caplength;
-	register u_int i;
-	register int s1, s2;
-	register int nshorts;
+	u_int i;
+	u_int s1, s2;
+	u_int nshorts;
 	char hexstuff[HEXDUMP_SHORTS_PER_LINE*HEXDUMP_HEXSTUFF_PER_SHORT+1], *hsp;
 	char asciistuff[ASCII_LINELENGTH+1], *asp;
 
-	caplength = (ndo->ndo_snapend >= cp) ? ndo->ndo_snapend - cp : 0;
+	caplength = (ndo->ndo_snapend > cp) ? ND_BYTES_AVAILABLE_AFTER(cp) : 0;
 	if (length > caplength)
 		length = caplength;
 	nshorts = length / sizeof(u_short);
 	i = 0;
 	hsp = hexstuff; asp = asciistuff;
-	while (--nshorts >= 0) {
-		s1 = *cp++;
-		s2 = *cp++;
+	while (nshorts != 0) {
+		s1 = GET_U_1(cp);
+		cp++;
+		s2 = GET_U_1(cp);
+		cp++;
 		(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
 		    " %02x%02x", s1, s2);
 		hsp += HEXDUMP_HEXSTUFF_PER_SHORT;
-		*(asp++) = (ND_ISGRAPH(s1) ? s1 : '.');
-		*(asp++) = (ND_ISGRAPH(s2) ? s2 : '.');
+		*(asp++) = (char)(ND_ASCII_ISGRAPH(s1) ? s1 : '.');
+		*(asp++) = (char)(ND_ASCII_ISGRAPH(s2) ? s2 : '.');
 		i++;
 		if (i >= HEXDUMP_SHORTS_PER_LINE) {
 			*hsp = *asp = '\0';
-			ND_PRINT((ndo, "%s0x%04x: %-*s  %s",
+			ND_PRINT("%s0x%04x: %-*s  %s",
 			    ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
-			    hexstuff, asciistuff));
+			    hexstuff, asciistuff);
 			i = 0; hsp = hexstuff; asp = asciistuff;
 			oset += HEXDUMP_BYTES_PER_LINE;
 		}
+		nshorts--;
 	}
 	if (length & 1) {
-		s1 = *cp++;
+		s1 = GET_U_1(cp);
+		cp++;
 		(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
 		    " %02x", s1);
 		hsp += 3;
-		*(asp++) = (ND_ISGRAPH(s1) ? s1 : '.');
+		*(asp++) = (char)(ND_ASCII_ISGRAPH(s1) ? s1 : '.');
 		++i;
 	}
 	if (i > 0) {
 		*hsp = *asp = '\0';
-		ND_PRINT((ndo, "%s0x%04x: %-*s  %s",
+		ND_PRINT("%s0x%04x: %-*s  %s",
 		     ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
-		     hexstuff, asciistuff));
+		     hexstuff, asciistuff);
 	}
 }
 
 void
-hex_and_ascii_print(netdissect_options *ndo, register const char *ident,
-    register const u_char *cp, register u_int length)
+hex_and_ascii_print(netdissect_options *ndo, const char *ident,
+    const u_char *cp, u_int length)
 {
 	hex_and_ascii_print_with_offset(ndo, ident, cp, length, 0);
 }
@@ -157,26 +167,29 @@ hex_print_with_offset(netdissect_options *ndo,
 		      u_int oset)
 {
 	u_int caplength;
-	register u_int i, s;
-	register int nshorts;
+	u_int i, s;
+	u_int nshorts;
 
-	caplength = (ndo->ndo_snapend >= cp) ? ndo->ndo_snapend - cp : 0;
+	caplength = (ndo->ndo_snapend > cp) ? ND_BYTES_AVAILABLE_AFTER(cp) : 0;
 	if (length > caplength)
 		length = caplength;
-	nshorts = (u_int) length / sizeof(u_short);
+	nshorts = length / sizeof(u_short);
 	i = 0;
-	while (--nshorts >= 0) {
+	while (nshorts != 0) {
 		if ((i++ % 8) == 0) {
-			ND_PRINT((ndo,"%s0x%04x: ", ident, oset));
+			ND_PRINT("%s0x%04x: ", ident, oset);
 			oset += HEXDUMP_BYTES_PER_LINE;
 		}
-		s = *cp++;
-		ND_PRINT((ndo," %02x%02x", s, *cp++));
+		s = GET_U_1(cp);
+		cp++;
+		ND_PRINT(" %02x%02x", s, GET_U_1(cp));
+		cp++;
+		nshorts--;
 	}
 	if (length & 1) {
 		if ((i % 8) == 0)
-			ND_PRINT((ndo,"%s0x%04x: ", ident, oset));
-		ND_PRINT((ndo," %02x", *cp));
+			ND_PRINT("%s0x%04x: ", ident, oset);
+		ND_PRINT(" %02x", GET_U_1(cp));
 	}
 }
 
