@@ -22,10 +22,10 @@
 /* \summary: IPv6 fragmentation header printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
 #include "netdissect.h"
 #include "extract.h"
@@ -33,38 +33,34 @@
 #include "ip6.h"
 
 int
-frag6_print(netdissect_options *ndo, register const u_char *bp, register const u_char *bp2)
+frag6_print(netdissect_options *ndo, const u_char *bp, const u_char *bp2)
 {
-	register const struct ip6_frag *dp;
-	register const struct ip6_hdr *ip6;
+	const struct ip6_frag *dp;
+	const struct ip6_hdr *ip6;
 
+	ndo->ndo_protocol = "frag6";
 	dp = (const struct ip6_frag *)bp;
 	ip6 = (const struct ip6_hdr *)bp2;
 
-	ND_TCHECK(*dp);
-
 	if (ndo->ndo_vflag) {
-		ND_PRINT((ndo, "frag (0x%08x:%d|%ld)",
-		       EXTRACT_32BITS(&dp->ip6f_ident),
-		       EXTRACT_16BITS(&dp->ip6f_offlg) & IP6F_OFF_MASK,
-		       sizeof(struct ip6_hdr) + EXTRACT_16BITS(&ip6->ip6_plen) -
-			       (long)(bp - bp2) - sizeof(struct ip6_frag)));
+		ND_PRINT("frag (0x%08x:%u|%zu)",
+			 GET_BE_U_4(dp->ip6f_ident),
+			 GET_BE_U_2(dp->ip6f_offlg) & IP6F_OFF_MASK,
+			 sizeof(struct ip6_hdr) + GET_BE_U_2(ip6->ip6_plen) -
+			        (bp - bp2) - sizeof(struct ip6_frag));
 	} else {
-		ND_PRINT((ndo, "frag (%d|%ld)",
-		       EXTRACT_16BITS(&dp->ip6f_offlg) & IP6F_OFF_MASK,
-		       sizeof(struct ip6_hdr) + EXTRACT_16BITS(&ip6->ip6_plen) -
-			       (long)(bp - bp2) - sizeof(struct ip6_frag)));
+		ND_PRINT("frag (%u|%zu)",
+		         GET_BE_U_2(dp->ip6f_offlg) & IP6F_OFF_MASK,
+		         sizeof(struct ip6_hdr) + GET_BE_U_2(ip6->ip6_plen) -
+			         (bp - bp2) - sizeof(struct ip6_frag));
 	}
 
 	/* it is meaningless to decode non-first fragment */
-	if ((EXTRACT_16BITS(&dp->ip6f_offlg) & IP6F_OFF_MASK) != 0)
+	if ((GET_BE_U_2(dp->ip6f_offlg) & IP6F_OFF_MASK) != 0)
 		return -1;
 	else
 	{
-		ND_PRINT((ndo, " "));
+		ND_PRINT(" ");
 		return sizeof(struct ip6_frag);
 	}
-trunc:
-	ND_PRINT((ndo, "[|frag]"));
-	return -1;
 }
