@@ -38,21 +38,15 @@
  * Per RFC 768, September, 1981.
  */
 struct udphdr {
-	uint16_t	uh_sport;		/* source port */
-	uint16_t	uh_dport;		/* destination port */
-	uint16_t	uh_ulen;		/* udp length */
-	uint16_t	uh_sum;			/* udp checksum */
+	nd_uint16_t	uh_sport;		/* source port */
+	nd_uint16_t	uh_dport;		/* destination port */
+	nd_uint16_t	uh_ulen;		/* udp length */
+	nd_uint16_t	uh_sum;			/* udp checksum */
 };
 
 #ifndef NAMESERVER_PORT
 #define NAMESERVER_PORT			53
 #endif
-#ifndef TACACS_DB_PORT
-#define TACACS_DB_PORT			65	/*XXX*/
-#endif
-#ifndef ORACLE_SQLNET_PORT
-#define ORACLE_SQLNET_PORT		66	/*XXX*/
-#endif
 #ifndef BOOTPS_PORT
 #define BOOTPS_PORT			67	/* RFC951 */
 #endif
@@ -77,41 +71,17 @@ struct udphdr {
 #ifndef NETBIOS_DGRAM_PORT
 #define NETBIOS_DGRAM_PORT		138	/* RFC 1001, RFC 1002 */
 #endif
-#ifndef NETBIOS_SSN_PORT
-#define NETBIOS_SSN_PORT		139	/* RFC 1001, RFC 1002 */
-#endif
 #ifndef SNMP_PORT
 #define SNMP_PORT			161	/*XXX*/
 #endif
 #ifndef SNMPTRAP_PORT
 #define SNMPTRAP_PORT			162	/*XXX*/
 #endif
-#ifndef BGP_PORT
-#define BGP_PORT			179	/*XXX*/
+#ifndef PTP_EVENT_PORT
+#define PTP_EVENT_PORT	        	319 /* IANA */
 #endif
-#ifndef APPLETALK_RTMP_PORT
-#define APPLETALK_RTMP_PORT		201	/*XXX*/
-#endif
-#ifndef APPLETALK_NB_PORT
-#define APPLETALK_NB_PORT		202	/*XXX*/
-#endif
-#ifndef APPLETALK_ECHO
-#define APPLETALK_ECHO			204	/*XXX*/
-#endif
-#ifndef APPLETALK_ZONE_INFO_PORT
-#define APPLETALK_ZONE_INFO_PORT	206	/*XXX*/
-#endif
-#ifndef LDAP_PORT
-#define LDAP_PORT			389	/*XXX*/
-#endif
-#ifndef HTTPS_PORT
-#define HTTPS_PORT			443	/*XXX*/
-#endif
-#ifndef MICROSOFT_DS_PORT
-#define MICROSOFT_DS_PORT		445	/*XXX*/
-#endif
-#ifndef KERBEROS5_PASSWD_PORT
-#define KERBEROS5_PASSWD_PORT		464	/* PER IANA */
+#ifndef PTP_GENERAL_PORT
+#define PTP_GENERAL_PORT	        320 /* IANA */
 #endif
 #ifndef CISCO_AUTORP_PORT
 #define CISCO_AUTORP_PORT		496	/*XXX*/
@@ -131,27 +101,15 @@ struct udphdr {
 #ifndef TIMED_PORT
 #define TIMED_PORT			525	/*XXX*/
 #endif
-#ifndef KERBEROS_LOGIN_PORT
-#define KERBEROS_LOGIN_PORT		543	/*XXX*/
-#endif
-#ifndef KERBEROS_SHELL_PORT
-#define KERBEROS_SHELL_PORT		544	/*XXX*/
-#endif
 #ifndef DHCP6_SERV_PORT
 #define DHCP6_SERV_PORT			546	/*XXX*/
 #endif
 #ifndef DHCP6_CLI_PORT
 #define DHCP6_CLI_PORT			547	/*XXX*/
 #endif
-#ifndef LDAPS_PORT
-#define LDAPS_PORT			636	/*XXX - LDAP over TLS/SSL */
-#endif
 #ifndef LDP_PORT
 #define LDP_PORT			646
 #endif
-#ifndef DHCP_FAILOVER_PORT
-#define DHCP_FAILOVER_PORT		647	/*XXX*/
-#endif
 #ifndef AQDV_PORT
 #define AODV_PORT			654	/*XXX*/
 #endif
@@ -161,36 +119,12 @@ struct udphdr {
 #ifndef LMP_PORT
 #define LMP_PORT			701	/* rfc4204 */
 #endif
-#ifndef CISCO_TDP_PORT
-#define CISCO_TDP_PORT			711	/*XXX*/
-#endif
-#ifndef KERBEROS_ADM_PORT
-#define KERBEROS_ADM_PORT		749	/*XXX - Kerberos v5 */
-#endif
 #ifndef KERBEROS_SEC_PORT
 #define KERBEROS_SEC_PORT		750	/*XXX - Kerberos v4 */
 #endif
-#ifndef RSYNC_PORT
-#define RSYNC_PORT			873	/*XXX*/
-#endif
 #ifndef LWRES_PORT
 #define LWRES_PORT			921	/*XXX*/
 #endif
-#ifndef OPENSSL_PORT
-#define OPENSSL_PORT			1194	/*XXX*/
-#endif
-#ifndef LOTUS_NOTES_PORT
-#define LOTUS_NOTES_PORT		1352	/*XXX*/
-#endif
-#ifndef MS_SQL_SERVER_PORT
-#define MS_SQL_SERVER_PORT		1433	/*XXX*/
-#endif
-#ifndef MS_SQL_SERVER_MONITOR
-#define MS_SQL_SERVER_MONITOR		1434	/*XXX*/
-#endif
-#ifndef INGRESLOCK_PORT
-#define INGRESLOCK_PORT			1524	/*XXX*/
-#endif
 #ifndef VQP_PORT
 #define VQP_PORT			1589	/*XXX*/
 #endif
@@ -215,42 +149,30 @@ struct udphdr {
 #ifndef HSRP_PORT
 #define HSRP_PORT			1985	/*XXX*/
 #endif
-#ifndef NFS_DAEMON_PORT
-#define NFS_DAEMON_PORT			2049	/*XXX*/
-#endif
 #ifndef ZEPHYR_SRV_PORT
 #define ZEPHYR_SRV_PORT			2103	/*XXX*/
 #endif
 #ifndef ZEPHYR_CLI_PORT
 #define ZEPHYR_CLT_PORT			2104	/*XXX*/
 #endif
-#ifndef MYSQL_PORT
-#define MYSQL_PORT			3306	/*XXX*/
-#endif
-#ifndef MS_RDP_PORT
-#define MS_RDP_PORT			3389	/*XXX*/
-#endif
 #ifndef VAT_PORT
 #define VAT_PORT			3456	/*XXX*/
 #endif
 #ifndef MPLS_LSP_PING_PORT
 #define MPLS_LSP_PING_PORT		3503	/* draft-ietf-mpls-lsp-ping-02.txt */
 #endif
-#ifndef SUBVERSION_PORT
-#define SUBVERSION_PORT			3690	/*XXX*/
+#ifndef BCM_LI_PORT
+#define BCM_LI_PORT			49152   /* SDK default */
 #endif
 #ifndef BFD_CONTROL_PORT
-#define BFD_CONTROL_PORT		3784	/* draft-katz-ward-bfd-v4v6-1hop-00.txt */
+#define BFD_CONTROL_PORT		3784	/* RFC 5881 */
 #endif
 #ifndef BFD_ECHO_PORT
-#define BFD_ECHO_PORT			3785	/* draft-katz-ward-bfd-v4v6-1hop-00.txt */
+#define BFD_ECHO_PORT			3785	/* RFC 5881 */
 #endif
 #ifndef RADIUS_COA_PORT
 #define RADIUS_COA_PORT			3799	/* RFC 5176 */
 #endif
-#ifndef NFS_LOCK_DAEMON_PORT
-#define NFS_LOCK_DAEMON_PORT		4045	/*XXX*/
-#endif
 #ifndef LISP_CONTROL_PORT
 #define LISP_CONTROL_PORT		4342	/* RFC 6830 */
 #endif
@@ -260,15 +182,15 @@ struct udphdr {
 #ifndef WB_PORT
 #define WB_PORT				4567
 #endif
+#ifndef BFD_MULTIHOP_PORT
+#define BFD_MULTIHOP_PORT		4784	/* RFC 5883 */
+#endif
 #ifndef VXLAN_PORT
 #define VXLAN_PORT			4789	/* RFC 7348 */
 #endif
 #ifndef VXLAN_GPE_PORT
 #define VXLAN_GPE_PORT			4790	/* draft-ietf-nvo3-vxlan-gpe-01 */
 #endif
-#ifndef SIP_DS_PORT
-#define SIP_DS_PORT			5059	/*XXX*/
-#endif
 #ifndef SIP_PORT
 #define SIP_PORT			5060
 #endif
@@ -282,7 +204,10 @@ struct udphdr {
 #define GENEVE_PORT			6081	/* draft-gross-geneve-02 */
 #endif
 #ifndef SFLOW_PORT
-#define SFLOW_PORT			6343	/* http://www.sflow.org/developers/specifications.php */
+#define SFLOW_PORT			6343	/* https://sflow.org/developers/specifications.php */
+#endif
+#ifndef MPLS_PORT
+#define MPLS_PORT			6635	/* RFC 7510 */
 #endif
 #ifndef BABEL_PORT
 #define BABEL_PORT			6696	/* RFC 6126 errata */
@@ -290,6 +215,9 @@ struct udphdr {
 #ifndef BABEL_PORT_OLD
 #define BABEL_PORT_OLD			6697	/* RFC 6126 */
 #endif
+#ifndef BFD_LAG_PORT
+#define BFD_LAG_PORT			6784	/* RFC 7310 */
+#endif
 #ifndef RX_PORT_LOW
 #define RX_PORT_LOW			7000	/*XXX*/
 #endif
@@ -314,3 +242,9 @@ struct udphdr {
 #ifndef LWAPP_CONTROL_PORT
 #define LWAPP_CONTROL_PORT		12223	/* RFC 5412 */
 #endif
+#ifndef ZEP_PORT
+#define ZEP_PORT			17754	/* XXX */
+#endif
+#ifndef SOMEIP_PORT
+#define SOMEIP_PORT			30490	/* https://www.autosar.org/standards/foundation */
+#endif
